﻿''' <summary>コンピュータ</summary>
Public Class CPU

    Private GridBoard As GridBoard
    Private Status As CellStatus
    Private Enemy As CellStatus
    Private VirtualBoard As GridBoard

    '探索深度
    Public SEARCH_LEVEL As Integer = 5

    Public Sub New(ByVal GridBoard As GridBoard, ByVal Status As CellStatus)

        Me.GridBoard = GridBoard
        If Status = CellStatus.Black Then
            Me.Status = CellStatus.Black
            Me.Enemy = CellStatus.White
        Else
            Me.Status = CellStatus.White
            Me.Enemy = CellStatus.Black
        End If

        Me.VirtualBoard = GridBoard.Copy()

    End Sub

    Public Sub Put()

        'CPUが石を置くメソッド

        Dim temp As Integer = AlphaBeta(True, SEARCH_LEVEL, Integer.MinValue, Integer.MaxValue)

        Dim X As Integer = temp Mod GridBoard.Cell_X
        Dim Y As Integer = temp / GridBoard.Cell_X

        If GridBoard.CanPut(Status, X, Y) Then
            GridBoard.Cells(X, Y).Hint = True 'HintをTrue
            Return
        Else
            '予備
            For X2 As Integer = 0 To GridBoard.Cell_X - 1
                For Y2 As Integer = 0 To GridBoard.Cell_Y - 1
                    If GridBoard.CanPut(Status, X2, Y2) Then
                        GridBoard.Cells(X, Y).Hint = True 'HintをTrue
                        Return
                    End If
                Next
            Next
        End If

    End Sub

    'α-β探索
    Private Function AlphaBeta(ByVal Flag As Boolean, ByVal Level As Integer, ByVal Alpha As Integer, ByVal Beta As Integer) As Integer

        '仮想ボード
        Dim Board As GridBoard = VirtualBoard.Copy()

        'ノードの評価値
        Dim Value As Integer

        '子ノードからの評価値
        Dim childValue As Integer

        '最大評価値
        Dim bestX As Integer = 0
        Dim bestY As Integer = 0

        'ゲームの木の末端は盤面評価
        If (Level = 0) Then
            Return BoardValue(Me.Status)
        End If

        '初期化
        If Flag Then
            '自分の手番では最大の評価値を見つけたいから、最初に最小値をセット
            Value = Integer.MinValue

            'もしパスの場合は盤面評価値を返す
            If VirtualBoard.PuttableCount(Me.Status) = 0 Then
                Return BoardValue(Me.Status)
            End If
        Else
            '相手の手番では最小の評価値を見つけたいから、最初に最大値をセット
            Value = Integer.MaxValue
            'もしパスの場合は盤面評価値を返す
            If VirtualBoard.PuttableCount(Me.Enemy) = 0 Then
                Return BoardValue(Me.Enemy)
            End If
        End If

        '打てるところはすべて試す
        For X As Integer = 0 To GridBoard.Cell_X - 1
            For Y As Integer = 0 To GridBoard.Cell_Y - 1
                If Flag Then
                    If VirtualBoard.CanPut(Me.Status, X, Y) Then

                        '試しに打つ
                        VirtualBoard.Put(Me.Status, X, Y)

                        '子ノードの評価値
                        childValue = AlphaBeta(Not Flag, Level - 1, Alpha, Beta)

                        If childValue > Value Then
                            Value = childValue

                            'α値更新
                            Alpha = Value
                            bestX = X
                            bestY = Y
                        End If

                        If Value > Beta Then
                            'βカット
                            VirtualBoard = Board.Copy()
                            Return Value
                        End If

                    End If
                Else
                    If VirtualBoard.CanPut(Me.Enemy, X, Y) Then

                        '試しに打つ
                        VirtualBoard.Put(Me.Enemy, X, Y)

                        '子ノードの評価値
                        childValue = AlphaBeta(Not Flag, Level - 1, Alpha, Beta)

                        If childValue < Value Then
                            Value = childValue

                            'β値更新
                            Beta = Value
                            bestX = X
                            bestY = Y
                        End If

                        If Value < Alpha Then
                            'αカット
                            VirtualBoard = Board.Copy()
                            Return Value
                        End If

                    End If
                End If
            Next
        Next

        'Return 前に VirtualBoard = Board.Copy()
        If Level = SEARCH_LEVEL Then
            VirtualBoard = Board.Copy()
            Return bestX + bestY * GridBoard.Cell_X
        Else
            VirtualBoard = Board.Copy()
            Return Value
        End If

    End Function

    Public Function BoardValue(ByVal Status As CellStatus) As Integer

        '評価値を格納する変数
        Dim point As Integer = 0

        '表の代入
        Dim CellValue(,) As Integer = {
            {120, -20, 20, 5, 5, 20, -20, 120},
            {-20, -40, -5, -5, -5, -5, -40, -20},
            {20, -5, 15, 3, 3, 15, -5, 20},
            {5, -5, 3, 3, 3, 3, -5, 5},
            {5, -5, 3, 3, 3, 3, -5, 5},
            {20, -5, 15, 3, 3, 15, -5, 20},
            {-20, -40, -5, -5, -5, -5, -40, -20},
            {120, -20, 20, 5, 5, 20, -20, 120}}
        '   120 -20 20  5  5 20 -20 120
        '   -20 -40 -5 -5 -5 -5 -40 -20
        '    20  -5 15  3  3 15  -5  20
        '     5  -5  3  3  3  3  -5   5
        '     5  -5  3  3  3  3  -5   5
        '    20  -5 15  3  3 15  -5  20
        '   -20 -40 -5 -5 -5 -5 -40 -20
        '   120 -20 20  5  5 20 -20 120


        Dim Enemy As CellStatus

        Select Case Status
            Case CellStatus.Black
                Enemy = CellStatus.White
            Case CellStatus.White
                Enemy = CellStatus.Black
        End Select

        Dim _cell As CellStatus

        Dim X As Integer = 0 'ループ変数
        Dim Y As Integer = 0 'ループ変数

        For X = 0 To GridBoard.Cell_X - 1
            For Y = 0 To GridBoard.Cell_Y - 1
                _cell = GridBoard.Cells(X, Y).Status

                Select Case _cell
                    Case Status
                        point += CellValue(X, Y)
                    Case Enemy
                        point -= CellValue(X, Y)
                End Select
            Next
        Next

        Return point

    End Function
End Class

