﻿''' <summary>オセロのグリッド(盤面)を表します。石を置くためのメソッドや、ひっくり返すメソッドとそれに関連するイベント、セルの管理、描画処理を行います</summary>
Public Class GridBoard

    '定数定義

    ''' <summary>セルサイズ</summary>
    Public Const CellSize As Integer = 68

    ''' <summary>Xセルの数</summary>
    Public Const Cell_X As Integer = 8

    ''' <summary>Yセルの数</summary>
    Public Const Cell_Y As Integer = 8

    ''' <summary>セル全体を表す配列</summary>
    Public CellMember(Cell_X - 1, Cell_Y - 1) As Cell

    ''' <summary>新しく石を置いたときに発生するイベント</summary>
    Public Event PutNew(ByVal sender As Object, ByVal e As EventArgs)

    ''' <summary>ひっくり返すときに発生するイベント</summary>
    Public Event Reverced(ByVal sender As Object, ByVal e As EventArgs)

    ''' <summary></summary>
    Public TurnNum As Integer = 60

    ''' <summary>グリッドボードクラスのコンストラクタ。各セルに場所を与える(生成)</summary>
    Public Sub New()

        Dim X As Integer = 0 'ループ変数
        Dim Y As Integer = 0 'ループ変数

        For X = 0 To Cell_X - 1
            For Y = 0 To Cell_Y - 1
                CellMember(X, Y) = New Cell(Me, New Point(X, Y))
            Next
        Next

    End Sub

    ''' <summary>セルの位置を受け取り、セルを返す</summary>
    ''' <param name="X">X座標(0～7)</param>
    ''' <param name="Y">Y座標(0～7)</param>
    ''' <value>セルを返す</value>
    Public Property Cells(ByVal X As Integer, ByVal Y As Integer) As Cell
        Get
            Return CellMember(X, Y)
        End Get
        Set(ByVal value As Cell)
            CellMember(X, Y) = value
        End Set
    End Property

    ''' <summary>描画メソッド</summary>
    ''' <param name="g">グラフィックオブジェクト</param>
    Public Sub Draw(ByVal g As Graphics)

        Dim X As Integer = 0 'ループ変数
        Dim Y As Integer = 0 'ループ変数

        '線を引くペンの設定
        Dim LinePen As New Pen(Brushes.Black, 1)

        '背景を塗る
        g.FillRectangle(Brushes.YellowGreen, 0, 0, CellSize * Cell_X, CellSize * Cell_Y)

        '縦線
        For X = 0 To Cell_X + 1
            g.DrawLine(LinePen, CellSize * X, 0, CellSize * X, CellSize * Cell_Y)
        Next

        '横線
        For Y = 0 To Cell_Y + 1
            g.DrawLine(LinePen, 0, CellSize * Y, CellSize * Cell_X, CellSize * Y)
        Next

        '４つの点を打つ
        g.FillEllipse(Brushes.Black, CellSize * 2 - 4, CellSize * 2 - 4, 8, 8)  '左上
        g.FillEllipse(Brushes.Black, CellSize * 6 - 4, CellSize * 2 - 4, 8, 8)  '右上
        g.FillEllipse(Brushes.Black, CellSize * 2 - 4, CellSize * 6 - 4, 8, 8)  '左下
        g.FillEllipse(Brushes.Black, CellSize * 6 - 4, CellSize * 6 - 4, 8, 8)  '右下

        'セルの状態を描画()
        For X = 0 To Cell_X - 1
            For Y = 0 To Cell_Y - 1
                Cells(X, Y).Draw(g)
            Next
        Next

        'メモリ解放
        LinePen.Dispose()

    End Sub

    ''' <summary>指定座標にあるセルの取得</summary>
    ''' <param name="X">X座標を指定</param>
    ''' <param name="Y">Y座標を指定</param>
    Public Function CellPoint(ByVal X As Integer, ByVal Y As Integer) As Cell

        Dim HereCell As Cell

        If X < 0 OrElse X >= CellSize * Cell_X Then
            Return Nothing
        End If

        If Y < 0 OrElse Y >= CellSize * Cell_Y Then
            Return Nothing
        End If

        HereCell = Cells(X \ CellSize, Y \ CellSize)

        Return HereCell

    End Function

    ''' <summary>ひっくり返せる石の数を計測</summary>
    ''' <param name="Status">置こうとしてる石の色</param>
    ''' <param name="X">X座標指定</param>
    ''' <param name="Y">Y座標指定</param>
    ''' <returns>石を置いたとき、ひっくり返せる石の数を返します</returns>
    Public Overloads Function PieceFlip(ByVal Status As CellStatus, ByVal X As Integer, Y As Integer) As Integer

        'ひっくり返せる石の数を格納する変数
        Dim Number As Integer = 0

        Dim i As Integer = 0 'ループ変数

        For i = 0 To 7
            Number += PieceFlip(Status, i, X, Y)
        Next

        Return Number

    End Function

    ''' <summary>ひっくり返せる石の数を計測</summary>
    ''' <param name="Status">置こうとしてる石の色</param>
    ''' <param name="Exposure">方向</param>
   ''' <param name="X">X座標指定</param>
    ''' <param name="Y">Y座標指定</param>
    ''' <returns>石を置いたとき、ひっくり返せる石の数を返します</returns>
    Public Overloads Function PieceFlip(ByVal Status As CellStatus, ByVal Exposure As ScanExposure, ByVal X As Integer, ByVal Y As Integer) As Integer

        '相手の色の変数
        Dim Enemy As CellStatus

        '自分の色を下に相手の色を設定(自分はStatus , 相手はEnemy)
        Select Case Status
            Case CellStatus.Black
                Enemy = CellStatus.White
            Case CellStatus.White
                Enemy = CellStatus.Black
        End Select

        '方向を示す変数
        Dim LeftRight As Integer = 0  '左=-1,右=1
        Dim UpDowm As Integer = 0     '上=-1,下=1

        Select Case Exposure
            Case ScanExposure.Left       '左
                LeftRight = -1
                UpDowm = 0

            Case ScanExposure.Right      '右
                LeftRight = 1
                UpDowm = 0

            Case ScanExposure.Up         '上
                LeftRight = 0
                UpDowm = -1

            Case ScanExposure.Down       '下
                LeftRight = 0
                UpDowm = 1

            Case ScanExposure.LeftUp     '左上
                LeftRight = -1
                UpDowm = -1

            Case ScanExposure.LeftDown   '左下
                LeftRight = -1
                UpDowm = 1

            Case ScanExposure.RightUp    '右上
                LeftRight = 1
                UpDowm = -1

            Case ScanExposure.RightDown  '右下
                LeftRight = 1
                UpDowm = 1
        End Select

        Dim Check As Point
        Dim Number As Integer  'ひっくり返せる石の数を格納する変数
        Dim HereCell As Cell

        'この条件に当てはまれば、指定した座標は端っこ。ひっくり返せない
        If (X + LeftRight) < 0 OrElse (X + LeftRight) >= Cell_X OrElse (Y + UpDowm) < 0 OrElse (Y + UpDowm) >= Cell_Y Then
            Return 0 'Falseを返してもいい
        End If

            '次のセルを取得
            HereCell = Cells(X + LeftRight, Y + UpDowm)

            'その次のセルが相手の色なら、ひっくり返せるかも知れない
            If Enemy = Cells(X + LeftRight, Y + UpDowm).Status Then

                Dim i As Integer = 0 'ループ変数

                For i = 0 To Cell_X - 1

                    'その隣の座標を取得
                    Check.X = X + (LeftRight * (i + 1))
                    Check.Y = Y + (UpDowm * (i + 1))

                    '↑のセル(Checkに格納した座標の)がグリッド外だったら、ひっくり返せる石はない
                    If Check.X < 0 OrElse Check.X >= Cell_X Then
                        Return 0 'Falseを返してもいい
                    End If

                    If Check.Y < 0 OrElse Check.Y >= Cell_Y Then
                        Return 0 'Falseを返してもいい
                    End If

                    '↑のセル(Checkに格納した座標)の色が
                    '自分と同じ色ならひっくり返せる・何も無ければひっくり返せない・相手の色ならひっくり返せる可能性がある
                    Select Case Cells(Check.X, Check.Y).Status
                        Case Status
                            Return Number 'ひっくり返せる数を返す
                        Case CellStatus.Nothing
                            Return 0 'Falseを返してもいい
                        Case Enemy
                            Number += 1 'ひっくり返せる可能性がある
                    End Select

                Next

            End If

            Return 0 'Falseを返してもいい

    End Function

    ''' <summary>セルに石が置けるか判定</summary>
    ''' <param name="Turn">置こうとしている石の色(どっちのターンか)</param>
    ''' <param name="X">X座標指定</param>
    ''' <param name="Y">Y座標指定</param>
    ''' <returns>石が置けるかどうかをブール値で返す</returns>
    Public Function CanPut(ByVal Turn As CellStatus, ByVal X As Integer, ByVal Y As Integer) As Boolean

        '指定したセルに石が置かれていたら、新しくおけるはずがないのでFalseを返す
        '指定したセルに石を置いたとき、ひっくり返せる石がなければ置けないのでFalseを返す
        If Cells(X, Y).Status <> CellStatus.Nothing Or PieceFlip(Turn, X, Y) = 0 Then
            Return False
        End If

        Return True

    End Function


    ''' <summary>ひっくり返す</summary>
    ''' <param name="Status">置いた石の色</param>
    ''' <param name="Exposure">方向</param>
    ''' <param name="X">X座標を指定</param>
    ''' <param name="Y">Y座標を指定</param>
    Public Sub Reverce(ByVal Status As CellStatus, ByVal Exposure As Integer, ByVal X As Integer, ByVal Y As Integer)

        '相手の色の変数
        Dim Enemy As CellStatus

        '自分の色を下に相手の色を設定(自分はStatus , 相手はEnemy)
        Select Case Status
            Case CellStatus.Black
                Enemy = CellStatus.White
            Case CellStatus.White
                Enemy = CellStatus.Black
        End Select

        '方向を示す変数
        Dim LeftRight As Integer = 0  '左=-1,右=1
        Dim UpDowm As Integer = 0     '上=-1,下=1

        Select Case Exposure
            Case ScanExposure.Left       '左
                LeftRight = -1
                UpDowm = 0

            Case ScanExposure.Right      '右
                LeftRight = 1
                UpDowm = 0

            Case ScanExposure.Up         '上
                LeftRight = 0
                UpDowm = -1

            Case ScanExposure.Down       '下
                LeftRight = 0
                UpDowm = 1

            Case ScanExposure.LeftUp     '左上
                LeftRight = -1
                UpDowm = -1

            Case ScanExposure.LeftDown   '左下
                LeftRight = -1
                UpDowm = 1

            Case ScanExposure.RightUp    '右上
                LeftRight = 1
                UpDowm = -1

            Case ScanExposure.RightDown  '右下
                LeftRight = 1
                UpDowm = 1
        End Select

        Dim Check As Point

        'ひっくり返す
        If PieceFlip(Status, Exposure, X, Y) > 0 Then

            Dim i As Integer = 0 'ループ変数

            For i = 1 To Cell_X - 1
                Check.X = X + (LeftRight * i)
                Check.Y = Y + (UpDowm * i)

                '↑のセル(Checkに格納した座標の)がグリッド外だったら、ひっくり返せる石はない
                If Check.X < 0 OrElse Check.X >= Cell_X Then
                    Exit For
                End If

                If Check.Y < 0 OrElse Check.Y >= Cell_Y Then
                    Exit For
                End If

                If Cells(Check.X, Check.Y).Status = Enemy Then
                    Cells(Check.X, Check.Y).Status = Status
                    RaiseEvent Reverced(Me, New EventArgs)
                Else
                    Exit For
                End If

            Next

        End If

    End Sub

    ''' <summary>石を置きます</summary>
    ''' <param name="Status">石の色</param>
    ''' <param name="X">置く石のX座標</param>
    ''' <param name="Y">置く石のY座標</param>
    ''' <returns>置けたかどうか</returns>
    Public Function Put(ByVal Status As CellStatus, ByVal X As Integer, ByVal Y As Integer) As Boolean

        If X >= 8 Or Y >= 8 Then
            MsgBox("put1")
            MsgBox(New StackFrame(2).GetMethod.Name)
        End If

        '石が置けるか判定
        If CanPut(Status, X, Y) = False Then
            Return False
        End If

        '石を置く
        Cells(X, Y).Focus()
        Cells(X, Y).Status = Status

        'イベント発生
        RaiseEvent PutNew(Me, New EventArgs)

        Dim i As Integer = 0 'ループ変数

        For i = 0 To 7
            Reverce(Status, i, X, Y)
        Next

        TurnNum -= 1

        Return True

    End Function

    ''' <summary>石の数を数える</summary>
    ''' <param name="Status">色の指定</param>
    ''' <returns>石の数を返します</returns>
    Public ReadOnly Property Count(ByVal Status As CellStatus)
        Get
            Dim HereCount As Integer = 0

            For Each Cell As Cell In CellMember
                If Cell.Status = Status Then
                    HereCount += 1
                End If
            Next

            Return HereCount
        End Get
    End Property

    ''' <summary>置ける場所の数を数える</summary>
    ''' <param name="Status">調べる色</param>
    ''' <returns>置ける場所の数を返す</returns>
    Public Function PuttableCount(ByVal Status As CellStatus) As Integer

        Dim Number As Integer = 0

        For Each Cell As Cell In CellMember
            If CanPut(Status, Cell.CellPosition.X, Cell.CellPosition.Y) Then
                Number += 1
            End If
        Next

        Return Number

    End Function

    ''' <summary>指定した座標に石を置いたとき、ある座標の石を返せるか</summary>
    ''' <param name="Status">色を指定</param>
    ''' <param name="X1">置く石のX座標</param>
    ''' <param name="Y1">置く石のY座標</param>
    ''' <param name="X2">ある石のX座標</param>
    ''' <param name="Y2">ある石のY座標</param>
    ''' <returns>返せるかをブール値で返す</returns>
    Public Function PointReverce(ByVal Status As CellStatus, ByVal X1 As Integer, ByVal Y1 As Integer, ByVal X2 As Integer, ByVal Y2 As Integer) As Boolean

        '相手の色の変数
        Dim Enemy As CellStatus

        '自分の色を下に相手の色を設定(自分はStatus , 相手はEnemy)
        Select Case Status
            Case CellStatus.Black
                Enemy = CellStatus.White
            Case CellStatus.White
                Enemy = CellStatus.Black
        End Select

        'ひっくり返せるかも知れないフラグ
        Dim flag As Boolean = False

        Dim Exposure As Integer = 0 'ループ変数

        '方向を示す変数
        Dim LeftRight As Integer = 0  '左=-1,右=1
        Dim UpDowm As Integer = 0     '上=-1,下=1

        For Exposure = 0 To 8

            Select Case Exposure
                Case ScanExposure.Left       '左
                    LeftRight = -1
                    UpDowm = 0

                Case ScanExposure.Right      '右
                    LeftRight = 1
                    UpDowm = 0

                Case ScanExposure.Up         '上
                    LeftRight = 0
                    UpDowm = -1

                Case ScanExposure.Down       '下
                    LeftRight = 0
                    UpDowm = 1

                Case ScanExposure.LeftUp     '左上
                    LeftRight = -1
                    UpDowm = -1

                Case ScanExposure.LeftDown   '左下
                    LeftRight = -1
                    UpDowm = 1

                Case ScanExposure.RightUp    '右上
                    LeftRight = 1
                    UpDowm = -1

                Case ScanExposure.RightDown  '右下
                    LeftRight = 1
                    UpDowm = 1
            End Select

            Dim Check As Point
            Dim HereCell As Cell

            'この条件に当てはまれば、指定した座標は端っこ。ひっくり返せない
            If (X1 + LeftRight) < 0 OrElse (X1 + LeftRight) >= Cell_X OrElse (Y1 + UpDowm) < 0 OrElse (Y1 + UpDowm) >= Cell_Y Then
                Return False
            End If

            '次のセルを取得
            HereCell = Cells(X1 + LeftRight, Y1 + UpDowm)

            'その次のセルが相手の色なら、ひっくり返せるかも知れない
            If Enemy = Cells(X1 + LeftRight, Y1 + UpDowm).Status Then

                Dim i As Integer = 0 'ループ変数

                Dim Possibility As Boolean = False 'ひっくり返せる可能性

                For i = 0 To Cell_X - 1

                    'その隣の座標を取得
                    Check.X = X1 + (LeftRight * (i + 1))
                    Check.Y = Y1 + (UpDowm * (i + 1))

                    '↑のセル(Checkに格納した座標の)がグリッド外だったら、ひっくり返せる石はない
                    If Check.X < 0 OrElse Check.X >= Cell_X Then
                        'ひっくりかえす石はない
                        Exit For
                    End If

                    If Check.Y < 0 OrElse Check.Y >= Cell_Y Then
                        'ひっくりかえす石はない
                        Exit For
                    End If


                    '↑のセル(Checkに格納した座標)の色が
                    '自分と同じ色ならひっくり返せる・何も無ければひっくり返せない・相手の色ならひっくり返せる可能性がある
                    Select Case Cells(Check.X, Check.Y).Status
                        Case Status
                            If Check.X = X2 And Check.Y = Y2 Or Possibility Then
                                flag = True
                            End If
                        Case CellStatus.Nothing

                        Case Enemy
                            'ひっくり返せる可能性がある
                            If Check.X = X2 And Check.Y = Y2 Then
                                Possibility = True
                            End If
                    End Select

                Next

            End If

        Next

        'ひっくり返せるか
        If flag = True Then
            Return True
        End If

        Return False

    End Function

    ''' <summary>現在のボードのコピーを作る</summary>
    Public Function Copy() As GridBoard

        Dim VirtualBoard As New GridBoard
        Dim X As Integer = 0 'ループ変数
        Dim Y As Integer = 0 'ループ変数

        For X = 0 To Cell_X - 1
            For Y = 0 To Cell_Y - 1
                VirtualBoard.Cells(X, Y).Status = Cells(X, Y).Status
            Next
        Next

        Return VirtualBoard

    End Function

End Class
