﻿Imports System.Net

Public Class MainForm

    ''' <summary>GridBoardのインスタンス。オセロのグリッド(盤面)を表します。石を置くためのメソッドや、ひっくり返すメソッドとそれに関連するイベント、セルの管理、描画処理を行います</summary>
    Private WithEvents GridBoard As New GridBoard

    'どっちの色のターン
    Private Turn As CellStatus = CellStatus.Black

    'Playerの色
    Private PlayersColor As CellStatus

    'Enemyの色
    Private EnemyColor As CellStatus

    Private _SendClass As New SendClass
    Private threads1 As New System.Threading.Thread(AddressOf Receive)
    Private UdpClient As New System.Net.Sockets.UdpClient(6000) '受信するポート番号
    Private RemoteIpEndPoint As New System.Net.IPEndPoint(System.Net.IPAddress.Any, 0)
    Private str As String = ""
    Private LoopFlag As Boolean = True
    Private ConnectFlag = False
    Private ConnectOption(2) As String

    'Server or Client　とポート番号と接続先アドレス
    Private Sub MainForm_Load(sender As Object, e As EventArgs) Handles MyBase.Load

        Dim addressList As IPAddress() = Dns.GetHostAddresses(Dns.GetHostName())

        Dim f As StartForm = New StartForm
        ConnectOption = f.ShowDialog()

        If ConnectOption(0) = "server" Then
            '自分がサーバー
            'SendClass初期化
            _SendClass.Main(ConnectOption(1))
            ToolStripStatusLabel1.Text = "Connecting to " & ConnectOption(1) & "    "
            PlayersColor = CellStatus.Black
            EnemyColor = CellStatus.White
            _SendClass.SendMsg("00" & addressList(4).ToString())
        Else
            '敵がサーバー
            PlayersColor = CellStatus.White
            EnemyColor = CellStatus.Black
            ToolStripStatusLabel1.Text = "Not Connecting" & "    "
        End If

        'MsgBox(ConnectOption(0) & vbNewLine & ConnectOption(1) ) 'テスト

        PictureBox1.Enabled = False

        threads1.Start()
        Timer1.Enabled = True

    End Sub

    ''' <summary>初期設定</summary>
    Private Sub Init()

        '初期設定
        For Each Cell As Cell In GridBoard.CellMember
            Cell.Status = CellStatus.Nothing
        Next

        '初期配置
        GridBoard.Cells(3, 3).Status = CellStatus.White
        GridBoard.Cells(4, 4).Status = CellStatus.White
        GridBoard.Cells(3, 4).Status = CellStatus.Black
        GridBoard.Cells(4, 3).Status = CellStatus.Black

        'ピクチャーボックスの再描画
        PictureBox1.Invalidate()

        'TurnPanelの可視設定
        pctBlackTurn.Visible = True
        pctWhiteTurn.Visible = False

        '現在の石の数を表示
        lblBlackCount.Text = GridBoard.Count(CellStatus.Black)
        lblWhiteCount.Text = GridBoard.Count(CellStatus.White)

        'TurnNumの初期化
        GridBoard.TurnNum = 60

        'ピクチャーボックスの有効化
        PictureBox1.Enabled = True

        '白ターンにして、ターン交代をする（＝黒ターン開始)
        Turn = CellStatus.White

        TurnChange()

        If PlayersColor = CellStatus.Black Then
            Puttable()
        End If

    End Sub

    Private counter As Integer = 0
    Private Sub Timer1_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer1.Tick
        Timer1.Enabled = False

        'まだ接続がされてない
        If ConnectOption(0) = "server" Then
            If Not ConnectFlag Then
                If counter >= 30 Then
                    Dim addressList As IPAddress() = Dns.GetHostAddresses(Dns.GetHostName())
                    _SendClass.SendMsg("00" & addressList(4).ToString())
                    counter = 0
                Else
                    counter += 1
                End If
            End If
        End If

        If str = "" Then
        Else
            Select Case str.Substring(0, 2)
                Case "00" '00が送られた＝自分はclient
                    If Not ConnectFlag Then
                        _SendClass.Main(str.Substring(2)) '送られてきたデータ00は相手のAddress
                        ToolStripStatusLabel1.Text = "Connecting to " & str.Substring(2) & "    "
                        ConnectFlag = True
                        Init() '初期化
                    End If
                    _SendClass.SendMsg("10") '自分は接続確認を送る

                Case "10" '01が送られた＝自分はserver
                    ConnectFlag = True
                    Init()

                Case "01" '相手が置いた石の座標
                    If GridBoard.Put(Turn, Integer.Parse(str.Substring(2, 1)), Integer.Parse(str.Substring(3, 1))) Then
                        TurnChange()
                    End If

                Case "02" 'Meassage受信
                    MsgBox(str.Substring(2))

                Case "03" '接続終了
                    MsgBox("接続が切断されました" & vbNewLine & "Othelloを終了します")
                    Environment.Exit(0) '強制終了
            End Select
            str = ""
        End If
        Timer1.Enabled = True
    End Sub

    Private Sub Receive()
        Do
            Try
                Dim receiveBytes As [Byte]() = UdpClient.Receive(RemoteIpEndPoint)
                '↑受信するまでこの行でとまっています。
                Dim returnData As String = System.Text.Encoding.Default.GetString(receiveBytes)

                str = returnData

            Catch err As Exception
                MsgBox(err.ToString)

            End Try
        Loop While (LoopFlag)
    End Sub

    Private Sub PictureBox1_Paint(sender As Object, e As PaintEventArgs) Handles PictureBox1.Paint

        'グリッドボードの描画
        GridBoard.Draw(e.Graphics)

    End Sub

    ''' <summary>クリックした場所を(X,Y)に変換して石を置く</summary>
    Private Sub PictureBox1_Click(sender As Object, e As EventArgs) Handles PictureBox1.Click

        Dim ClickPosition As Point 'クリックしたセルの座標を格納する変数

        Dim HereCell As Cell

        ClickPosition = PictureBox1.PointToClient(Windows.Forms.Cursor.Position) 'クリックした座標を変数に格納

        HereCell = GridBoard.CellPoint(ClickPosition.X, ClickPosition.Y)

        If GridBoard.Put(Turn, HereCell.CellPosition.X, HereCell.CellPosition.Y) Then

            _SendClass.SendMsg("01" & HereCell.CellPosition.X & HereCell.CellPosition.Y)
            TurnChange()

        End If

    End Sub

    ''' <summary>マウスが動いたとき、フォーカスの設定</summary>
    Private Sub PictureBox1_MouseMove(sender As Object, e As MouseEventArgs) Handles PictureBox1.MouseMove

        Dim HereCell As Cell

        'マウスがあるセルを取得
        HereCell = GridBoard.CellPoint(e.X, e.Y)

        If Not IsNothing(HereCell) Then

            'セルが取得できたらセルにフォーカスの枠を表示
            HereCell.Focus()

            'ピクチャーボックスの再描画
            PictureBox1.Invalidate()

        End If

    End Sub

    ''' <summary>ピクチャーボックスからマウスが離れたとき全てのセルのフォーカスを外す</summary>
    Private Sub PictureBox1_MouseLeave(sender As Object, e As EventArgs) Handles PictureBox1.MouseLeave

        Dim X As Integer = 0 'ループ変数
        Dim Y As Integer = 0 'ループ変数

        'すべてのセルのフォーカスを解除
        For X = 0 To GridBoard.Cell_X - 1
            For Y = 0 To GridBoard.Cell_Y - 1
                GridBoard.Cells(X, Y).Focused = False
            Next
        Next

        'ピクチャーボックスの再描画
        PictureBox1.Invalidate()

    End Sub

    Public Sub TurnChange()

        'ピクチャーボックスの再描画
        PictureBox1.Invalidate()

        If GridBoard.Count(CellStatus.Nothing) = 0 Then
            '置ける場所がない場合
            Result()
            Return
        ElseIf GridBoard.PuttableCount(CellStatus.Black) = 0 AndAlso GridBoard.PuttableCount(CellStatus.White) = 0 Then
            '黒も白も置けない場合
            Result()
            Return
        ElseIf GridBoard.Count(CellStatus.Black) = 0 Then
            '黒全滅の場合
            MsgBox("Winner White!!", 0, "Perfect Winner")
            pctWhiteTurn.Visible = True
            pctBlackTurn.Visible = False
            Return
        ElseIf GridBoard.Count(CellStatus.White) = 0 Then
            '白全滅の場合
            MsgBox("Winner Black!!", 0, "Perfect Winner")
            pctBlackTurn.Visible = True
            pctWhiteTurn.Visible = False
            Return
        End If

        '次のターンを決める
        Select Case Turn
            Case CellStatus.Black
                Turn = CellStatus.White
                pctWhiteTurn.Visible = True
                pctBlackTurn.Visible = False
            Case CellStatus.White
                Turn = CellStatus.Black
                pctBlackTurn.Visible = True
                pctWhiteTurn.Visible = False
        End Select

        '置ける場所があるか判定
        If GridBoard.PuttableCount(Turn) = 0 Then
            '置く場所がない場合はパスする
            MsgBox("Pass")
            TurnChange()
        End If

        '自分の番かEnemyの番か判定
        If Turn = PlayersColor Then
            '自分の番なら、ピクチャーボックスを使用可能にする
            PictureBox1.Enabled = True
            Puttable()
            ToolStripStatusLabel2.Text = "Your Turn"
        Else
            'Enemyの番なら、ピクチャーボックスを使用不可にする
            PictureBox1.Enabled = False
            unPuttable()
            unHint()
            'Enemyが石を置く
            ToolStripStatusLabel2.Text = "Enemy's Turn"
        End If

    End Sub

    '結果を表示する
    Private Sub Result()
        If GridBoard.Count(CellStatus.Black) > GridBoard.Count(CellStatus.White) Then
            MsgBox("Winner Black!!", 0, "Winner")
            pctBlackTurn.Visible = True
            pctWhiteTurn.Visible = False
        ElseIf GridBoard.Count(CellStatus.Black) < GridBoard.Count(CellStatus.White) Then
            MsgBox("Winner White!!", 0, "Winner")
            pctWhiteTurn.Visible = True
            pctBlackTurn.Visible = False
        Else
            MsgBox("Draw", 0, "Draw")
            pctWhiteTurn.Visible = True
            pctBlackTurn.Visible = True
        End If
        MsgBox("Othelloを終了します")
        Environment.Exit(0)
    End Sub

    ''' <summary>石を置いたときに発生するイベント</summary>
    Private Sub GridBoard_PutNew(ByVal sender As Object, ByVal e As System.EventArgs) Handles GridBoard.PutNew
        GridBoard_Recived(sender, e)
    End Sub

    ''' <summary>石がひっくり返されたときに発生するイベント</summary>
    Private Sub GridBoard_Recived(ByVal sender As Object, ByVale As System.EventArgs) Handles GridBoard.Reverced

        'ピクチャーボックスの再描画
        PictureBox1.Invalidate()

        '現在の石の数を表示
        lblBlackCount.Text = GridBoard.Count(CellStatus.Black)
        lblWhiteCount.Text = GridBoard.Count(CellStatus.White)

        '石を少しずつひっくり返す
        Application.DoEvents()
        Threading.Thread.Sleep(100)

    End Sub

    ''' <summary>ヒントを表示します</summary>
    Private Sub Puttable()

        Dim HereCell As Cell
        Dim X As Integer = 0 'ループ変数
        Dim Y As Integer = 0 'ループ変数

        For X = 0 To GridBoard.Cell_X - 1
            For Y = 0 To GridBoard.Cell_Y - 1

                GridBoard.Cells(X, Y).Puttabled = False

                HereCell = GridBoard.Cells(X, Y)

                If Not IsNothing(HereCell) Then
                    HereCell.Puttable(PlayersColor)
                End If

            Next
        Next

        'ピクチャーボックスの再描画
        PictureBox1.Invalidate()

    End Sub

    ''' <summary>ヒントを消します</summary>
    Private Sub unPuttable()

        Dim X As Integer
        Dim Y As Integer

        '全てのセルのヒントを消す
        For X = 0 To GridBoard.Cell_X - 1
            For Y = 0 To GridBoard.Cell_Y - 1
                GridBoard.Cells(X, Y).Puttabled = False
            Next
        Next

        '再描画
        PictureBox1.Invalidate()

    End Sub

#Region "透明度設定"
    Private Sub ToolStripMenuItem2_Click(sender As Object, e As EventArgs) Handles ToolStripMenuItem2.Click
        Me.Opacity = 1.0
    End Sub

    Private Sub ToolStripMenuItem3_Click(sender As Object, e As EventArgs) Handles ToolStripMenuItem3.Click
        Me.Opacity = 0.9
    End Sub

    Private Sub ToolStripMenuItem4_Click(sender As Object, e As EventArgs) Handles ToolStripMenuItem4.Click
        Me.Opacity = 0.8
    End Sub

    Private Sub ToolStripMenuItem5_Click(sender As Object, e As EventArgs) Handles ToolStripMenuItem5.Click
        Me.Opacity = 0.7
    End Sub

    Private Sub ToolStripMenuItem6_Click(sender As Object, e As EventArgs) Handles ToolStripMenuItem6.Click
        Me.Opacity = 0.6
    End Sub

    Private Sub ToolStripMenuItem7_Click(sender As Object, e As EventArgs) Handles ToolStripMenuItem7.Click
        Me.Opacity = 0.5
    End Sub

    Private Sub ToolStripMenuItem8_Click(sender As Object, e As EventArgs) Handles ToolStripMenuItem8.Click
        Me.Opacity = 0.4
    End Sub

    Private Sub ToolStripMenuItem9_Click(sender As Object, e As EventArgs) Handles ToolStripMenuItem9.Click
        Me.Opacity = 0.3
    End Sub

    Private Sub ToolStripMenuItem10_Click(sender As Object, e As EventArgs) Handles ToolStripMenuItem10.Click
        Me.Opacity = 0.2
    End Sub

    Private Sub ToolStripMenuItem11_Click(sender As Object, e As EventArgs) Handles ToolStripMenuItem11.Click
        Me.Opacity = 0.1
    End Sub

    Private Sub ExitEToolStripMenuItem_Click(sender As Object, e As EventArgs) Handles ExitEToolStripMenuItem.Click
        Me.Close()
    End Sub

#End Region

    'Messageを送る
    Private Sub MessageMToolStripMenuItem_Click(sender As Object, e As EventArgs) Handles MessageMToolStripMenuItem.Click
        If ConnectFlag Then
            _SendClass.SendMsg("02" & InputBox("Input Send Message", "Message", "Hello"))
        End If
    End Sub

    Private Sub MainForm_FormClosing(sender As Object, e As FormClosingEventArgs) Handles Me.FormClosing
        If MsgBox("Othelloを終了します", MsgBoxStyle.OkCancel) = MsgBoxResult.Cancel Then
            e.Cancel = True
        Else
            If ConnectFlag Then
                _SendClass.SendMsg("03")
                Application.DoEvents()
                Threading.Thread.Sleep(500)
            End If
            Environment.Exit(0)
        End If
    End Sub

    'ヒント
    Private Sub ToolStripMenuItem1_Click(sender As Object, e As EventArgs) Handles ToolStripMenuItem1.Click
        'ヒント(最善手？)呼び出し  
        Dim cpu As New CPU(GridBoard, PlayersColor)
        cpu.Put()
        PictureBox1.Invalidate()
    End Sub

    Private Sub unHint()
        Dim X As Integer
        Dim Y As Integer

        '全てのセルのヒントを消す
        For X = 0 To GridBoard.Cell_X - 1
            For Y = 0 To GridBoard.Cell_Y - 1
                GridBoard.Cells(X, Y).Hint = False
            Next
        Next
        PictureBox1.Invalidate()
    End Sub

End Class